; Name: ATTiny2313.asm
; LED1 & LED2 auf dem Pollin Atmel Evaluationboard blinken
; (c) Alfons Moeller Januar 2013
.INCLUDE "tn2313def.inc"           ;AVR-Definitionsdatei einbinden
.INCLUDE "zifres.asm"
.equ inpchar = var000       ; letztes mit der Tastatur eingegebenes Zeichen
.equ countchar = var002     ; in der linken unteren Zeile das erste Zeichen
.equ hcount = var003        ; Counter 1/100 sec
.equ z0rt = var004          ; Zeitzaehler 1/100 sec
.equ sr_inp_portd = var008  ; Puffer Eingaenge Port D
.equ sr_oup_portd = var009  ; Puffer Ausgaenge Port D
.equ intcount = var010      ; Interuptzaehler in ms
.equ wascii = var011        ; ASCII Zeichen
;    wascii = var011+1
;    wascii = var012  
;    wascii = var012+1
;    wascii = var013  
.equ fu00c = var014         ; Funktionsschritt Textausgabe
.equ merk00 = var015        ; Merker Textausgabe starten
.equ zyksec = var016        ; Anzahl der durchlaufenen Programmzyklen pro Sekunde
.equ zykzae = var017        ; Zyklenzaehler
.equ sekunde = var018       ; Sekundenzaehler ab Start
.equ fu10c = var021         ; Funktionsschritt fuer die LED1
.equ takt = 1000            ; alle xxxx ms ein Takt 

;**************************************************************************
;	Macro ldiw	(Load Immediate Word)
; 	Expects: @0-word register pair to load into, @1- constant to load
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Loads the constant word into the register. Can be used to load pointers.
;**************************************************************************
.macro	ldiw
	ldi	@0L, low(@1)
	ldi	@0H, high(@1)
.endmacro
;**************************************************************************
;	Macro ldsw	(Load from SRAM word)
; 	Expects: @0- register pair, @1- address
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Loads the word from the SRAM address. Words are stored MSB last.
;**************************************************************************
.macro	ldsw
	lds	@0H, @1 + 1
	lds	@0L, @1
.endmacro
;**************************************************************************
;	Macro addw	(Add Word)
; 	Expects: @0- register pair for result, @1 register pair
;	Uses:	
;	Words:	2
;	Cycles: 2
;
;	Adds the two words together.
;**************************************************************************
.macro	addw
	add	@0L, @1L
	adc	@0H, @1H
.endmacro
;**************************************************************************
; SRAM WORD mit einer 16 Bit Konstanten setzen
;**************************************************************************
.MACRO SMEMW ;(RdL:RdH, k)
    push r16
    ldi r16,LOW(@1)
    sts @0,r16
    ldi r16,HIGH(@1)
    sts @0+1,r16
    pop r16
.ENDMACRO
;**************************************************************************
; SRAM 16 Bit Wert + 1
;**************************************************************************
.MACRO INCM ; MEM + 1
    push r16
    lds r16,@0
   	inc r16
   	sts @0,r16
    brne PC+6
                 ;
    lds r16,@0+1
   	inc r16
  	sts @0+1,r16
    pop r16
.ENDMACRO
;**************************************************************************
; SRAM 16 Bit Wert - 1
;**************************************************************************
.MACRO DECM ; MEM - 1
    push r16
    lds r16,@0
   	dec r16
   	sts @0,r16
    inc  r16
    brne PC+6
                 ;
    lds r16,@0+1
   	dec r16
  	sts @0+1,r16
    pop r16
.ENDMACRO

.cseg 
.ORG     0x0000
 rjmp RESET ; Reset Handler
 reti       ; rjmp INT0 ; External Interrupt0 Handler
 reti       ; rjmp INT1 ; External Interrupt1 Handler
 reti       ; rjmp TIM1_CAPT ; Timer1 Capture Handler
 reti       ; rjmp TIM1_COMPA ; Timer1 CompareA Handler
 reti       ; rjmp TIM1_OVF ; Timer1 Overflow Handler
 reti       ; rjmp TIM0_OVF ; Timer0 Overflow Handler
 reti       ; rjmp USART0_RXC ; USART0 RX Complete Handler
 reti       ; rjmp USART0_DRE ; USART0,UDR Empty Handler
 reti       ; rjmp USART0_TXC ; USART0 TX Complete Handler
 reti       ; rjmp ANA_COMP ; Analog Comparator Handler
 reti       ; rjmp PCINT ; Pin Change Interrupt
 reti       ; rjmp TIMER1_COMPB ; Timer1 Compare B Handler
 rjmp TIMER0_COMPA ; Timer0 Compare A Handler
 reti       ; rjmp TIMER0_COMPB ; Timer0 Compare B Handler
 reti       ; rjmp USI_START ; USI Start Handler
 reti       ; rjmp USI_OVERFLOW ; USI Overflow Handler
 reti       ; rjmp EE_READY ; EEPROM Ready Handler
 reti       ; rjmp WDT_OVERFLOW ; Watchdog Overflow Handler.org	0x13
;
RESET:
    ldi r16, low(RAMEND); Main program start
    out SPL,r16         ; Set Stack Pointer to top of RAM
    rcall USART_Init
    rcall TCNT0_Init
    sbi	ddrd,5
    sbi	ddrd,6
    ldi r16,'='
    sts inpchar,r16
    ldi r16,48
    sts countchar,r16
    ldi r16,low(takt)
	sts intcount,r16
    ldi r16,high(takt)
	sts intcount+1,r16
    ldi r16,low(takt/100)
    sts hcount,r16
    ldi r16,high(takt/100)
    sts hcount+1,r16
    ldi r16,0
    sts fu00c,r16
    sts fu00c+1,r16
    sts fu10c,r16
    sts fu10c+1,r16
    sts z0rt,r16
    sts sr_oup_portd,r16
    sts sr_inp_portd,r16
    sts sr_oup_portd+1,r16
    sts sr_inp_portd+1,r16
    sts sekunde,r16
    sts sekunde+1,r16
    sts sr_oup_portd+1,r16
    sei                 ; Enable interrupts
;=======================================
; Arbeiten
;=======================================
start1:
    rcall zyklus
    rjmp start1

zyklus:
    rcall st00
    rcall st10
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall andere_routinen
    rcall zeich_holen
    rcall inp_oup
    rcall schleife
    rcall taste_gedrueckt
    incm zykzae
    ret
;=======================================
; Ausgabe und Eingabe der Ein-Ausgnge
;=======================================
inp_oup:
    in  r11,pind
    sts sr_inp_portd,r11
    lds r11,sr_oup_portd
	out portd,r11    
    ret
;=======================================
; LED1 aus
;=======================================
led1_aus:
    lds r18,sr_oup_portd
	andi r18,(127-32)
    sts sr_oup_portd,r18
    ret
;=======================================
; LED1 ein
;=======================================
led1_ein:
    lds r18,sr_oup_portd
	ori r18,32
    sts sr_oup_portd,r18
    ret
;=======================================
; LED2 aus
;=======================================
led2_aus:
    lds r18,sr_oup_portd
	andi r18,(127-64)
    sts sr_oup_portd,r18
    ret
;=======================================
; LED2 ein
;=======================================
led2_ein:
    lds r18,sr_oup_portd
    ori r18,64
    sts sr_oup_portd,r18
    ret
;
;
;=======================================
; Schleife Beschftigungsprogramm fr einen Prozessor
; dient nur zur Demonstration von Prozessorzeit
; in diesem Beispiel: m1*m2*m3 = 100 mal pro Zyklus durchlaufen
;=======================================
;
;
.equ m1 = 10 ; innere Schleife
.equ m2 = 5  ; mittlere Schleife
.equ m3 = 2  ; uere Schleife
;
;
schleife:
 ldi r16,m1
 ldi r17,m2
 ldi r18,m3
 ldi r19,0
; uere Schleife
schleife3:
 ldi r17,m2
; mittlere Schleife
schleife2:
 ldi r16,m1
; innere Schleife
schleife1:    
 nop
 subi r16,1
 cp r16,r19
 brne schleife1
;
 subi r17,1
 cp r17,r19
 brne schleife2
;
 subi r18,1
 cp r18,r19
 brne schleife3
;
 ret
;=======================================
; andere beliebigen Routinen dienen nur
; zur Programmbelastung und knnen
; entfernt werden
;=======================================
andere_routinen:
   nop
   ret
;
;
; USART initialisieren
;
;
USART_Init:
; Set baud rate
ldi r17,0
ldi r16,25
out UBRRH, r17
out UBRRL, r16
; Enable receiver and transmitter
ldi r16, (1<<RXEN)|(1<<TXEN)
out UCSRB,r16
; Set frame format: 8 data, 1 stop bit
ldi r16, (1<<USBS)|(3<<UCSZ0)
out UCSRC,r16
ret
;
;
;internen Interrupt TCNT0 initialisieren
;
;
TCNT0_Init:
        ldi     r16,0b10000010        ; ........... 
        out     TCCR0A,r16
        ldi     r16,0b11001011         ; Waveform CTC // Modus SysClk/64  
        out     TCCR0B,r16
        ldi     r16,124                ; genau 1 ms 125 = 1+124 ?????
 		out     OCR0A,r16
        ldi     r16,0b00000101          
        out     TIMSK,r16
        ret
;-------------------------------------------------------------------------------
;
;	Zeichen abholen
;
;-------------------------------------------------------------------------------

zeich_holen:
    sbis UCSRA,RXC
	ret
;
    in r16,UDR
    sts var000,r16
    ldi r16,0
	sts var000+1,r16
	ret
;-------------------------------------------------------------------------------
;
;	Timer 0 Ereignis A
;
;-------------------------------------------------------------------------------
TIMER0_COMPA:
    push r16
; 1/100 sekunden zaehlen
    decm hcount
    lds r16,hcount
    cpi r16,0
    brne e1_timer
;
    lds r16,hcount+1
    cpi r16,0
    brne e1_timer
;
    ldi r16,low(takt/100)
    sts hcount,r16
    ldi r16,high(takt/100)
    sts hcount+1,r16
;
    lds r16,z0rt
    cpi r16,0
	breq e1_timer 
;   
	dec r16
    sts z0rt,r16
e1_timer:
; 1 ms zaehlen
    decm intcount
    lds r16,intcount
;   dec r16
;	sts intcount,r16
    cpi r16,0
    brne ende_timer
;
    lds r16,intcount+1
    cpi r16,0
    brne ende_timer
;
    ldi r16,low(takt)
    sts intcount,r16
    ldi r16,high(takt)
    sts intcount+1,r16
;
    lds r16,zykzae
    sts zyksec,r16
    lds r16,zykzae+1
    sts zyksec+1,r16
    smemw zykzae,0
    ldi r16,1
    sts merk00,r16
    incm sekunde
ende_timer:
    pop r16
    reti
;-------------------------------------------------------------------------------
;
;	Taste gedrckt ?
;
;-------------------------------------------------------------------------------
taste_gedrueckt:
 rcall taster3_gdr
 ret

.equ Taster1 = 4
.equ Taster2 = 8
.equ Taster3 = 16
.equ Hupe = 128
;-------------------------------------------------------------------------------
;
;	Taster 3 abfragen und solange wie gedrckt LED2 schalten
;
;-------------------------------------------------------------------------------
taster3_gdr:
    lds r16,sr_inp_portd
	andi r16,Taster3
; 	breq k_taste1
;
    rjmp LED2_schalten
;
;
k_taste1:
    rjmp led2_aus
;
;-------------------------------------------------------------------------------
;	LED2 schalten  
;-------------------------------------------------------------------------------
;  
LED2_schalten:
    lds r16,intcount
    andi r16,64
    cpi r16,64
	brne end_led2
;
    rjmp led2_ein
;
;
end_led2:
    rjmp led2_aus
;  
;-------------------------------------------------------------------------------
;	Terminal ansteuern mit Funktionseinheit 00
;-------------------------------------------------------------------------------
;  
st00:
    ldiw Z,ft00
    ldsw X,fu00c
    addw Z,X
    ijmp

ft00: rjmp warte      ; F000 warten bis Sendeauftrag
      rjmp wandeln    ; F001 umwandel von bin nach acii
      rjmp printA     ; F002 A anzeigen 
      rjmp printT     ; F003 T anzeigen
      rjmp printi     ; F004 i anzeigen
      rjmp printn     ; F005 n anzeigen
      rjmp printy     ; F006 y anzeigen
      rjmp print_     ; F007 _ anzeigen
      rjmp print2     ; F008 2 anzeigen
      rjmp print3     ; F009 3 anzeigen
      rjmp print1     ; F010 1 anzeigen
      rjmp print3     ; F011 3 anzeigen
      rjmp print_     ; F012 _ anzeigen
      rjmp printLF    ; F013 LF anzeigen
      rjmp printCR    ; F014 CR anzeigen
      rjmp printVAR   ; F015 VAR anzeigen
      rjmp printINP   ; F016 letztes eingegebenes anzeigen
      rjmp printINP   ; F017 letztes eingegebenes anzeigen
      rjmp printINP   ; F018 letztes eingegebenes anzeigen
      rjmp printINP   ; F019 letztes eingegebenes anzeigen
      rjmp printINP   ; F020 letztes eingegebenes anzeigen
      rjmp print1ZYK  ; F021 10000 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print2ZYK  ; F022 1000 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print3ZYK  ; F023 100 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print4ZYK  ; F024 10 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print5ZYK  ; F025 1 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print_     ; F026 _ anzeigen
      rjmp wandeln1   ; F027 umwandel von bin nach acii
      rjmp print1ZYK  ; F028 10000 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print2ZYK  ; F029 1000 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print3ZYK  ; F030 100 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print4ZYK  ; F031 10 er Stelle de Zyklen pro Sekunde darstellen
      rjmp print5ZYK  ; F032 1 er Stelle de Zyklen pro Sekunde darstellen
      rjmp printLF    ; F033 Linefeed
      rjmp printCR    ; F034 Return
      rjmp zaehle     ; F035 Sekunden zaehlen
      rjmp ful00      ; F036 Fuktionsschritt loeschen
;
;
; Fuktionsschritt erhoehen
;
;
inf00:
    incm fu00c
	ret
;
;
; Fuktionsschritt loeschen 
;
;
ful00:
    smemw fu00c,0
    ret
;
;
; USART Zeichen senden
;
;
USART_send:
; Wait for empty transmit buffer
  sbis UCSRA,UDRE
  ret
; Put data (r16) into buffer, sends the data
  out UDR,r16
  rjmp inf00

warte:
    lds r16,merk00
    cpi r16,1
    breq inf00     
;
    ret
;
; Zyklenzahl/sec BIN nach ASC wandeln
;
wandeln:
    lds rBin1L,zyksec  
    lds rBin1H,zyksec+1
    ldiw Z,wascii
    rcall Bin2ToAsc5
    rjmp inf00     
;
; Sekundenzaehler BIN nach ASC wandeln
;
wandeln1:
    lds rBin1L,sekunde
    lds rBin1H,sekunde+1
    ldiw Z,wascii
    rcall Bin2ToAsc5
    rjmp inf00     
;
printA:
ldi r16,'A'
 rjmp USART_send

printT:
ldi r16,'T'
 rjmp USART_send

printi:
ldi r16,'i'
 rjmp USART_send

printn:
ldi r16,'n'
 rjmp USART_send

printy:
ldi r16,'y'
 rjmp USART_send

print2:
ldi r16,'2'
 rjmp USART_send

print3:
ldi r16,'3'
 rjmp USART_send

print1:
ldi r16,'1'
 rjmp USART_send

print_:
ldi r16,' '
 rjmp USART_send

printLF:
ldi r16,10
 rjmp USART_send

printCR:
ldi r16,13
 rjmp USART_send

printVAR:
 lds r16,countchar
 rjmp USART_send

print1ZYK:
 lds r16,wascii
 rjmp USART_send

print2ZYK:
 lds r16,wascii+1
 rjmp USART_send

print3ZYK:
 lds r16,wascii+2
 rjmp USART_send  

print4ZYK:
 lds r16,wascii+3
 rjmp USART_send

print5ZYK:
 lds r16,wascii+4
 rjmp USART_send


printHZYK:
 lds r16,zyksec+1
 ldi r17,48
 add r16,r17
 rjmp USART_send

printINP:
 lds r16,inpchar
 rjmp USART_send

zaehle:
    lds r16,countchar
	inc r16
	cpi r16,58
	brne e_zaehle
;
    ldi r16,48
e_zaehle:
    sts countchar,r16
    ldi r16,0
    sts merk00,r16
  rjmp inf00
;  
;-------------------------------------------------------------------------------
;	LED 1 ansteuern mit Funktionseinheit 10
;-------------------------------------------------------------------------------
;  
st10:
    ldiw Z,ft10
    ldsw X,fu10c
    addw Z,X
    ijmp

ft10: rjmp LED1_xein  ; F000 LED1 einschalten
      rjmp zg0        ; F001 warten bis Zeit gleich 0
      rjmp LED1_xaus  ; F002 LED1 ausschalten
      rjmp zg0        ; F003 warten bis Zeit gleich 0
      rjmp ful10      ; F004 Funktionsschritt loeschen
inf10:
    incm fu10c
	ret
ful10:
    smemw fu10c,0
	ret

zg0:
    lds r16,z0rt  
    cpi r16,0
    breq inf10     
;
    ret
;=======================================
; LED1 ein 
;=======================================
LED1_xein:
    rcall led1_ein
    ldi r16,50
	sts z0rt,r16
    rjmp inf10
;=======================================
; LED1 aus
;=======================================
LED1_xaus:
    rcall led1_aus
    ldi r16,50
	sts z0rt,r16
    rjmp inf10
;
;
;
.DEF rmp = R16 ; als Vielzweckregister verwendet
.DEF rBin1L = R18
.DEF rBin1H = R19
.DEF rBin2L = R20
.DEF rBin2H = R21
;
; ************************************************
;
; Paket II: Von Binaer nach ASCII bzw. BCD
;
; Bin2ToAsc5
; ==========
; wandelt eine 16-Bit-Binaerzahl in eine fuenfstellige ASCII-
;   kodierte Dezimalzahl um
; Aufruf: 16-Bit-Binaerzahl in rBin1H:L, Z zeigt auf Anfang
;   der Zahl
; Rueckkehr: Z zeigt auf Anfang der Zahl, fuehrende Nullen sind
;   mit Leerzeichen ueberschrieben
; Benutzte Register: rBin1H:L (bleibt erhalten), rBin2H:L
;   (wird ueberschrieben), rmp
; Aufgerufene Unterroutinen: Bin2ToBcd5
;
Bin2ToAsc5:
	rcall Bin2ToBcd5 ; wandle Binaerzahl in BCD um
	ldi rmp,4 ; Zaehler auf 4
	mov rBin2L,rmp
Bin2ToAsc5a:
	ld rmp,z ; Lese eine BCD-Ziffer
	tst rmp ; pruefe ob Null
	brne Bin2ToAsc5b ; Nein, erste Ziffer <> 0 gefunden
	ldi rmp,' ' ; mit Leerzeichen ueberschreiben
	st z+,rmp ; und ablegen
	dec rBin2L ; Zaehler um eins senken
	brne Bin2ToAsc5a ; weitere fuehrende Leerzeichen
	ld rmp,z ; Lese das letzte Zeichen
Bin2ToAsc5b:
	inc rBin2L ; Ein Zeichen mehr
Bin2ToAsc5c:
	subi rmp,-'0' ; Addiere ASCII-0
	st z+,rmp ; und speichere ab, erhoehe Zeiger
	ld rmp,z ; naechstes Zeichen lesen
	dec rBin2L ; noch Zeichen behandeln?
	brne Bin2ToAsc5c ; ja, weitermachen
	sbiw ZL,5 ; Zeiger an Anfang
	ret ; fertig
;
; Bin2ToAsc
; =========
; wandelt eine 16-Bit-Binaerzahl in eine fuenfstellige ASCII-
;   kodierte Dezimalzahl um, Zeiger zeigt auf die erste signi-
;   fikante Ziffer der Zahl, und gibt Anzahl der Ziffern zu-
;   rueck
; Aufruf: 16-Bit-Binaerzahl in rBin1H:L, Z zeigt auf Anfang
;   der Zahl (5 Stellen erforderlich, auch bei kleineren Zah-
;   len!)
; Rueckkehr: Z zeigt auf erste signifikante Ziffer der ASCII-
;   kodierten Zahl, rBin2L enthaelt Laenge der Zahl (1..5)
; Benutzte Register: rBin1H:L (bleibt erhalten), rBin2H:L
;   (wird ueberschrieben), rmp
; Aufgerufene Unterroutinen: Bin2ToBcd5, Bin2ToAsc5
;
Bin2ToAsc:
	rcall Bin2ToAsc5 ; Wandle Binaerzahl in ASCII
	ldi rmp,6 ; Zaehler auf 6
	mov rBin2L,rmp
Bin2ToAsca:
	dec rBin2L ; verringere Zaehler
	ld rmp,z+ ; Lese Zeichen und erhoehe Zeiger
	cpi rmp,' ' ; war Leerzeichen?
	breq Bin2ToAsca ; Nein, war nicht
	sbiw ZL,1 ; ein Zeichen rueckwaerts
	ret ; fertig
;
; Bin2ToBcd5
; ==========
; wandelt 16-Bit-Binaerzahl in 5-stellige BCD-Zahl um
; Aufruf: 16-Bit-Binaerzahl in rBin1H:L, Z zeigt auf die
;   erste Stelle der BCD-kodierten Resultats
; Stellen: Die BCD-Zahl hat exakt 5 gueltige Stellen.
; Rueckkehr: Z zeigt auf die hoechste BCD-Stelle
; Benoetigte Register: rBin1H:L (wird erhalten), rBin2H:L
;   (wird nicht wieder hergestellt), rmp
; Aufgerufene Unterroutinen: Bin2ToDigit
;
Bin2ToBcd5:
	push rBin1H ; Rette Inhalt der Register rBin1H:L
	push rBin1L
	ldi rmp,HIGH(10000) ; Lade 10.000 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(10000)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 5.Stelle durch Abziehen
	ldi rmp,HIGH(1000) ; Lade 1.000 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(1000)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 4.Stelle durch Abziehen
	ldi rmp,HIGH(100) ; Lade 100 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(100)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 3.Stelle durch Abziehen
	ldi rmp,HIGH(10) ; Lade 10 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(10)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 2.Stelle durch Abziehen
	st z,rBin1L ; Rest sind Einer
	sbiw ZL,4 ; Setze Zeiger Z auf 5.Stelle (erste Ziffer)
	pop rBin1L ; Stelle den Originalwert wieder her
	pop rBin1H
	ret ; und kehre zurueck
;
; Bin2ToDigit
; ===========
; ermittelt eine dezimale Ziffer durch fortgesetztes Abziehen
; einer binaer kodierten Dezimalstelle
; Unterroutine benutzt von: Bin2ToBcd5, Bin2ToAsc5, Bin2ToAsc
; Aufruf: Binaerzahl in rBin1H:L, binaer kodierte Dezimalzahl
;   in rBin2H:L, Z zeigt auf bearbeitete BCD-Ziffer
; Rueckkehr: Ergebis in Z (bei Aufruf), Z um eine Stelle er-
;   hoeht, keine Fehlerbehandlung
; Benutzte Register: rBin1H:L (enthaelt Rest der Binaerzahl),
;   rBin2H (bleibt erhalten), rmp
; Aufgerufene Unterroutinen: -
;
Bin2ToDigit:
	clr rmp ; Zaehler auf Null
Bin2ToDigita:
	cp rBin1H,rBin2H ; Vergleiche MSBs miteinander
	brcs Bin2ToDigitc ; MSB Binaerzahl kleiner, fertig
	brne Bin2ToDigitb ; MSB Binaerzahl groesser, subtrahiere
	cp rBin1L,rBin2L ; MSB gleich, vergleiche LSBs
	brcs Bin2ToDigitc ; LSB Binaerzahl kleiner, fertig
Bin2ToDigitb:
	sub rBin1L,rBin2L ; Subtrahiere LSB Dezimalzahl
	sbc rBin1H,rBin2H ; Subtrahiere Carry und MSB
	inc rmp ; Erhoehe den Zaehler
	rjmp Bin2ToDigita ; Weiter vergleichen/subtrahieren
Bin2ToDigitc:
	st z+,rmp ; Speichere das Ergebnis und erhoehe Zeiger
	ret ; zurueck
